@@ -33,22 +33,19 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <gconf/gconf-client.h>
-
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-lowlevel.h>
 
 #include "gnome-settings-profile.h"
 #include "gsd-marshal.h"
 #include "gsd-media-keys-manager.h"
-#include "gsd-media-keys-manager-glue.h"
 
 #include "eggaccelerators.h"
 #include "acme.h"
 #include "gsd-media-keys-window.h"
+#include "gsd-input-helper.h"
 
 #ifdef HAVE_PULSE
 #include <canberra-gtk.h>
@@ -60,7 +57,22 @@
 #define GSD_MEDIA_KEYS_DBUS_PATH GSD_DBUS_PATH "/MediaKeys"
 #define GSD_MEDIA_KEYS_DBUS_NAME GSD_DBUS_NAME ".MediaKeys"
 
-#define TOUCHPAD_ENABLED_KEY "/desktop/gnome/peripherals/touchpad/touchpad_enabled"
+static const gchar introspection_xml[] =
+"<node>"
+"  <interface name='org.gnome.SettingsDaemon.MediaKeys'>"
+"    <annotation name='org.freedesktop.DBus.GLib.CSymbol' value='gsd_media_keys_manager'/>"
+"    <method name='GrabMediaPlayerKeys'>"
+"      <arg name='application' direction='in' type='s'/>"
+"      <arg name='time' direction='in' type='u'/>"
+"    </method>"
+"    <method name='ReleaseMediaPlayerKeys'>"
+"      <arg name='application' direction='in' type='s'/>"
+"    </method>"
+"    <signal name='MediaPlayerKeyPressed'/>"
+"  </interface>"
+"</node>";
+
+#define TOUCHPAD_ENABLED_KEY "touchpad-enabled"
 
 #define VOLUME_STEP 6           /* percents for one volume button press */
 #define MAX_VOLUME 65536.0
@@ -69,7 +81,9 @@
 
 typedef struct {
         char   *application;
+        char   *name;
         guint32 time;
+        guint   watch_id;
 } MediaPlayer;
 
 struct GsdMediaKeysManagerPrivate
@@ -80,7 +94,7 @@ struct GsdMediaKeysManagerPrivate
         GvcMixerStream  *stream;
 #endif /* HAVE_PULSE */
         GtkWidget       *dialog;
-        GConfClient     *conf_client;
+        GSettings       *settings;
         GVolumeMonitor  *volume_monitor;
 
         /* Multihead stuff */
@@ -89,17 +103,13 @@ struct GsdMediaKeysManagerPrivate
 
         GList           *media_players;
 
-        DBusGConnection *connection;
-        guint            notify[HANDLED_KEYS];
+        guint            owner_id;
+        GDBusNodeInfo   *introspection_data;
+        GDBusConnection *connection;
+        GDBusProxy      *xrandr_proxy;
+        GCancellable    *cancellable;
 };
 
-enum {
-        MEDIA_PLAYER_KEY_PRESSED,
-        LAST_SIGNAL
-};
-
-static guint signals[LAST_SIGNAL] = { 0 };
-
 static void     gsd_media_keys_manager_class_init  (GsdMediaKeysManagerClass *klass);
 static void     gsd_media_keys_manager_init        (GsdMediaKeysManager      *media_keys_manager);
 static void     gsd_media_keys_manager_finalize    (GObject                  *object);
@@ -152,25 +162,25 @@ acme_error (char * msg)
 static char *
 get_term_command (GsdMediaKeysManager *manager)
 {
-        char *cmd_term;
+        char *cmd_term, *cmd_args;;
         char *cmd = NULL;
+        GSettings *settings;
 
-        cmd_term = gconf_client_get_string (manager->priv->conf_client,
-                                            "/desktop/gnome/applications/terminal/exec", NULL);
-        if ((cmd_term != NULL) && (strcmp (cmd_term, "") != 0)) {
-                char *cmd_args;
-                cmd_args = gconf_client_get_string (manager->priv->conf_client,
-                                                    "/desktop/gnome/applications/terminal/exec_arg", NULL);
-                if ((cmd_args != NULL) && (strcmp (cmd_term, "") != 0)) {
-                        cmd = g_strdup_printf ("%s %s -e", cmd_term, cmd_args);
-                } else {
-                        cmd = g_strdup_printf ("%s -e", cmd_term);
-                }
+        settings = g_settings_new ("org.gnome.desktop.default-applications.terminal");
+        cmd_term = g_settings_get_string (settings, "exec");
+        if (cmd_term[0] == '\0')
+                cmd_term = g_strdup ("gnome-terminal");
 
-                g_free (cmd_args);
+        cmd_args = g_settings_get_string (settings, "exec-arg");
+        if (strcmp (cmd_term, "") != 0) {
+                cmd = g_strdup_printf ("%s %s -e", cmd_term, cmd_args);
+        } else {
+                cmd = g_strdup_printf ("%s -e", cmd_term);
         }
 
+        g_free (cmd_args);
         g_free (cmd_term);
+        g_object_unref (settings);
 
         return cmd;
 }
@@ -270,21 +280,20 @@ is_valid_shortcut (const char *string)
 }
 
 static void
-update_kbd_cb (GConfClient         *client,
-               guint                id,
-               GConfEntry          *entry,
+update_kbd_cb (GSettings           *settings,
+               const gchar         *key,
                GsdMediaKeysManager *manager)
 {
         int      i;
         gboolean need_flush = TRUE;
 
-        g_return_if_fail (entry->key != NULL);
+        g_return_if_fail (key != NULL);
 
         gdk_error_trap_push ();
 
         /* Find the key that was modified */
         for (i = 0; i < HANDLED_KEYS; i++) {
-                if (strcmp (entry->key, keys[i].gconf_key) == 0) {
+                if (strcmp (key, keys[i].settings_key) == 0) {
                         char *tmp;
                         Key  *key;
 
@@ -296,9 +305,10 @@ update_kbd_cb (GConfClient         *client,
                         g_free (keys[i].key);
                         keys[i].key = NULL;
 
-                        tmp = gconf_client_get_string (manager->priv->conf_client,
-                                                       keys[i].gconf_key, NULL);
+                        /* We can't have a change in a hard-coded key */
+                        g_assert (keys[i].settings_key != NULL);
 
+                        tmp = g_settings_get_string (manager->priv->settings, keys[i].settings_key);
                         if (is_valid_shortcut (tmp) == FALSE) {
                                 g_free (tmp);
                                 break;
@@ -341,17 +351,11 @@ init_kbd (GsdMediaKeysManager *manager)
                 char *tmp;
                 Key  *key;
 
-                manager->priv->notify[i] =
-                        gconf_client_notify_add (manager->priv->conf_client,
-                                                 keys[i].gconf_key,
-                                                 (GConfClientNotifyFunc) update_kbd_cb,
-                                                 manager,
-                                                 NULL,
-                                                 NULL);
-
-                tmp = gconf_client_get_string (manager->priv->conf_client,
-                                               keys[i].gconf_key,
-                                               NULL);
+                if (keys[i].settings_key != NULL) {
+                        tmp = g_settings_get_string (manager->priv->settings, keys[i].settings_key);
+                } else {
+                        tmp = g_strdup (keys[i].hard_coded);
+                }
 
                 if (!is_valid_shortcut (tmp)) {
                         g_debug ("Not a valid shortcut: '%s'", tmp);
@@ -452,121 +456,48 @@ dialog_show (GsdMediaKeysManager *manager)
 }
 
 static void
-do_unknown_action (GsdMediaKeysManager *manager,
-                   const char          *url)
+do_url_action (GsdMediaKeysManager *manager,
+               const char          *scheme)
 {
-        char *string;
+        GAppInfo *app_info;
 
-        g_return_if_fail (url != NULL);
+        app_info = g_app_info_get_default_for_uri_scheme (scheme);
+        if (app_info != NULL) {
+                GError *error = NULL;
 
-        string = gconf_client_get_string (manager->priv->conf_client,
-                                          "/desktop/gnome/url-handlers/unknown/command",
-                                          NULL);
-
-        if ((string != NULL) && (strcmp (string, "") != 0)) {
-                char *cmd;
-                cmd = g_strdup_printf (string, url);
-                execute (manager, cmd, FALSE, FALSE);
-                g_free (cmd);
-        }
-        g_free (string);
-}
-
-static void
-do_help_action (GsdMediaKeysManager *manager)
-{
-        char *string;
-
-        string = gconf_client_get_string (manager->priv->conf_client,
-                                          "/desktop/gnome/url-handlers/ghelp/command",
-                                          NULL);
-
-        if ((string != NULL) && (strcmp (string, "") != 0)) {
-                char *cmd;
-                cmd = g_strdup_printf (string, "");
-                execute (manager, cmd, FALSE, FALSE);
-                g_free (cmd);
+                if (!g_app_info_launch (app_info, NULL, NULL, &error)) {
+                        g_warning ("Could not launch '%s': %s",
+                                   g_app_info_get_commandline (app_info),
+                                   error->message);
+                        g_error_free (error);
+                }
         } else {
-                do_unknown_action (manager, "ghelp:");
+                g_warning ("Could not find default application for '%s' scheme", scheme);
         }
-
-        g_free (string);
-}
-
-static void
-do_mail_action (GsdMediaKeysManager *manager)
-{
-        char *string;
-
-        string = gconf_client_get_string (manager->priv->conf_client,
-                                          "/desktop/gnome/url-handlers/mailto/command",
-                                          NULL);
-
-        if ((string != NULL) && (strcmp (string, "") != 0)) {
-                char *cmd;
-                cmd = g_strdup_printf (string, "");
-                execute (manager,
-                         cmd,
-                         FALSE,
-                         gconf_client_get_bool (manager->priv->conf_client,
-                                                "/desktop/gnome/url-handlers/mailto/needs_terminal", NULL));
-                g_free (cmd);
-        }
-        g_free (string);
 }
 
 static void
 do_media_action (GsdMediaKeysManager *manager)
 {
-        char *command;
+        GAppInfo *app_info;
 
-        command = gconf_client_get_string (manager->priv->conf_client,
-                                           "/desktop/gnome/applications/media/exec", NULL);
-        if ((command != NULL) && (strcmp (command, "") != 0)) {
-                execute (manager,
-                         command,
-                         FALSE,
-                         gconf_client_get_bool (manager->priv->conf_client,
-                                                "/desktop/gnome/applications/media/needs_term", NULL));
-        }
-        g_free (command);
-}
+        app_info = g_app_info_get_default_for_type ("audio/ogg", FALSE);
+        if (app_info != NULL) {
+                GError *error = NULL;
 
-static void
-do_www_action (GsdMediaKeysManager *manager,
-               const char          *url)
-{
-        char *string;
-
-        string = gconf_client_get_string (manager->priv->conf_client,
-                                          "/desktop/gnome/url-handlers/http/command",
-                                          NULL);
-
-        if ((string != NULL) && (strcmp (string, "") != 0)) {
-                gchar *cmd;
-
-                if (url == NULL) {
-                        cmd = g_strdup_printf (string, "");
-                } else {
-                        cmd = g_strdup_printf (string, url);
+                if (!g_app_info_launch (app_info, NULL, NULL, &error)) {
+                        g_warning ("Could not launch '%s': %s",
+                                   g_app_info_get_commandline (app_info),
+                                   error->message);
+                        g_error_free (error);
                 }
-
-                execute (manager,
-                         cmd,
-                         FALSE,
-                         gconf_client_get_bool (manager->priv->conf_client,
-                                                "/desktop/gnome/url-handlers/http/needs_terminal", NULL));
-                g_free (cmd);
-        } else {
-                do_unknown_action (manager, url ? url : "");
         }
-        g_free (string);
 }
 
 static void
-do_exit_action (GsdMediaKeysManager *manager)
+do_logout_action (GsdMediaKeysManager *manager)
 {
-        execute (manager, "gnome-session-save --shutdown-dialog", FALSE, FALSE);
+        execute (manager, "gnome-session-save --logout", FALSE, FALSE);
 }
 
 static void
@@ -638,8 +569,18 @@ do_eject_action (GsdMediaKeysManager *manager)
 static void
 do_touchpad_action (GsdMediaKeysManager *manager)
 {
-        GConfClient *client = manager->priv->conf_client;
-        gboolean state = gconf_client_get_bool (client, TOUCHPAD_ENABLED_KEY, NULL);
+        GSettings *settings;
+        gboolean state;
+
+        if (touchpad_is_present () == FALSE) {
+                dialog_init (manager);
+                gsd_media_keys_window_set_action_custom (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
+                                                         "touchpad-disabled", FALSE);
+                return;
+        }
+
+        settings = g_settings_new ("org.gnome.settings-daemon.peripherals.touchpad");
+        state = g_settings_get_boolean (settings, TOUCHPAD_ENABLED_KEY);
 
         dialog_init (manager);
         gsd_media_keys_window_set_action_custom (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
@@ -647,7 +588,8 @@ do_touchpad_action (GsdMediaKeysManager *manager)
                                                  FALSE);
         dialog_show (manager);
 
-        gconf_client_set_bool (client, TOUCHPAD_ENABLED_KEY, !state, NULL);
+        g_settings_set_boolean (settings, TOUCHPAD_ENABLED_KEY, !state);
+        g_object_unref (settings);
 }
 
 #ifdef HAVE_PULSE
@@ -688,12 +630,7 @@ do_sound_action (GsdMediaKeysManager *manager,
         if (manager->priv->stream == NULL)
                 return;
 
-        vol_step = gconf_client_get_int (manager->priv->conf_client,
-                                         GCONF_MISC_DIR "/volume_step",
-                                         NULL);
-
-        if (vol_step <= 0 || vol_step > 100)
-                vol_step = VOLUME_STEP;
+        vol_step = VOLUME_STEP;
 
         norm_vol_step = PA_VOLUME_NORM * vol_step / 100;
 
@@ -796,6 +733,18 @@ on_control_default_sink_changed (GvcMixerControl     *control,
 
 #endif /* HAVE_PULSE */
 
+static void
+free_media_player (MediaPlayer *player)
+{
+        if (player->watch_id > 0) {
+                g_bus_unwatch_name (player->watch_id);
+                player->watch_id = 0;
+        }
+        g_free (player->application);
+        g_free (player->name);
+        g_free (player);
+}
+
 static gint
 find_by_application (gconstpointer a,
                      gconstpointer b)
@@ -803,6 +752,13 @@ find_by_application (gconstpointer a,
         return strcmp (((MediaPlayer *)a)->application, b);
 }
 
+static gint
+find_by_name (gconstpointer a,
+              gconstpointer b)
+{
+        return strcmp (((MediaPlayer *)a)->name, b);
+}
+
 static gint
 find_by_time (gconstpointer a,
               gconstpointer b)
@@ -810,6 +766,27 @@ find_by_time (gconstpointer a,
         return ((MediaPlayer *)a)->time < ((MediaPlayer *)b)->time;
 }
 
+static void
+name_vanished_handler (GDBusConnection     *connection,
+                       const gchar         *name,
+                       GsdMediaKeysManager *manager)
+{
+        GList *iter;
+
+        iter = g_list_find_custom (manager->priv->media_players,
+                                   name,
+                                   find_by_name);
+
+        if (iter != NULL) {
+                MediaPlayer *player;
+
+                player = iter->data;
+                g_debug ("Deregistering vanished %s (name: %s)", player->application, player->name);
+                free_media_player (player);
+                manager->priv->media_players = g_list_delete_link (manager->priv->media_players, iter);
+        }
+}
+
 /*
  * Register a new media player. Most applications will want to call
  * this with time = GDK_CURRENT_TIME. This way, the last registered
@@ -817,14 +794,15 @@ find_by_time (gconstpointer a,
  * may want to register with a lower priority (usually 1), to grab
  * events only nobody is interested.
  */
-gboolean
+static void
 gsd_media_keys_manager_grab_media_player_keys (GsdMediaKeysManager *manager,
                                                const char          *application,
-                                               guint32              time,
-                                               GError             **error)
+                                               const char          *name,
+                                               guint32              time)
 {
         GList       *iter;
         MediaPlayer *media_player;
+        guint        watch_id;
 
         if (time == GDK_CURRENT_TIME) {
                 GTimeVal tv;
@@ -839,45 +817,63 @@ gsd_media_keys_manager_grab_media_player_keys (GsdMediaKeysManager *manager,
 
         if (iter != NULL) {
                 if (((MediaPlayer *)iter->data)->time < time) {
-                        g_free (((MediaPlayer *)iter->data)->application);
-                        g_free (iter->data);
+                        MediaPlayer *player = iter->data;
+                        free_media_player (player);
                         manager->priv->media_players = g_list_delete_link (manager->priv->media_players, iter);
                 } else {
-                        return TRUE;
+                        return;
                 }
         }
 
+        watch_id = g_bus_watch_name (G_BUS_TYPE_SESSION,
+                                     name,
+                                     G_BUS_NAME_WATCHER_FLAGS_NONE,
+                                     NULL,
+                                     (GBusNameVanishedCallback) name_vanished_handler,
+                                     manager,
+                                     NULL);
+
         g_debug ("Registering %s at %u", application, time);
         media_player = g_new0 (MediaPlayer, 1);
         media_player->application = g_strdup (application);
+        media_player->name = g_strdup (name);
         media_player->time = time;
+        media_player->watch_id = watch_id;
 
         manager->priv->media_players = g_list_insert_sorted (manager->priv->media_players,
                                                              media_player,
                                                              find_by_time);
-
-        return TRUE;
 }
 
-gboolean
+static void
 gsd_media_keys_manager_release_media_player_keys (GsdMediaKeysManager *manager,
                                                   const char          *application,
-                                                  GError             **error)
+                                                  const char          *name)
 {
-        GList *iter;
+        GList *iter = NULL;
 
-        iter = g_list_find_custom (manager->priv->media_players,
-                                   application,
-                                   find_by_application);
+        g_return_if_fail (application != NULL || name != NULL);
 
-        if (iter != NULL) {
-                g_debug ("Deregistering %s", application);
-                g_free (((MediaPlayer *)iter->data)->application);
-                g_free (iter->data);
-                manager->priv->media_players = g_list_delete_link (manager->priv->media_players, iter);
+        if (application != NULL) {
+                iter = g_list_find_custom (manager->priv->media_players,
+                                           application,
+                                           find_by_application);
         }
 
-        return TRUE;
+        if (iter == NULL && name != NULL) {
+                iter = g_list_find_custom (manager->priv->media_players,
+                                           name,
+                                           find_by_name);
+        }
+
+        if (iter != NULL) {
+                MediaPlayer *player;
+
+                player = iter->data;
+                g_debug ("Deregistering %s (name: %s)", application, player->name);
+                free_media_player (player);
+                manager->priv->media_players = g_list_delete_link (manager->priv->media_players, iter);
+        }
 }
 
 static gboolean
@@ -886,6 +882,9 @@ gsd_media_player_key_pressed (GsdMediaKeysManager *manager,
 {
         const char *application = NULL;
         gboolean    have_listeners;
+        GError     *error = NULL;
+
+        g_debug ("Media key '%s' pressed", key);
 
         have_listeners = (manager->priv->media_players != NULL);
 
@@ -893,11 +892,57 @@ gsd_media_player_key_pressed (GsdMediaKeysManager *manager,
                 application = ((MediaPlayer *)manager->priv->media_players->data)->application;
         }
 
-        g_signal_emit (manager, signals[MEDIA_PLAYER_KEY_PRESSED], 0, application, key);
+        if (g_dbus_connection_emit_signal (manager->priv->connection,
+                                           NULL,
+                                           GSD_MEDIA_KEYS_DBUS_PATH,
+                                           GSD_MEDIA_KEYS_DBUS_NAME,
+                                           "MediaPlayerKeyPressed",
+                                           g_variant_new ("(ss)", application, key),
+                                           &error) == FALSE) {
+                g_debug ("Error emitting signal: %s", error->message);
+                g_error_free (error);
+        }
 
         return !have_listeners;
 }
 
+static void
+handle_method_call (GDBusConnection       *connection,
+                    const gchar           *sender,
+                    const gchar           *object_path,
+                    const gchar           *interface_name,
+                    const gchar           *method_name,
+                    GVariant              *parameters,
+                    GDBusMethodInvocation *invocation,
+                    gpointer               user_data)
+{
+        GsdMediaKeysManager *manager = (GsdMediaKeysManager *) user_data;
+
+        g_debug ("Calling method '%s' for media-keys", method_name);
+
+        if (g_strcmp0 (method_name, "ReleaseMediaPlayerKeys") == 0) {
+                const char *app_name;
+
+                g_variant_get (parameters, "(&s)", &app_name);
+                gsd_media_keys_manager_release_media_player_keys (manager, app_name, sender);
+                g_dbus_method_invocation_return_value (invocation, NULL);
+        } else if (g_strcmp0 (method_name, "GrabMediaPlayerKeys") == 0) {
+                const char *app_name;
+                guint32 time;
+
+                g_variant_get (parameters, "(&su)", &app_name, &time);
+                gsd_media_keys_manager_grab_media_player_keys (manager, app_name, sender, time);
+                g_dbus_method_invocation_return_value (invocation, NULL);
+        }
+}
+
+static const GDBusInterfaceVTable interface_vtable =
+{
+        handle_method_call,
+        NULL, /* Get Property */
+        NULL, /* Set Property */
+};
+
 static gboolean
 do_multimedia_player_action (GsdMediaKeysManager *manager,
                              const char          *key)
@@ -905,9 +950,86 @@ do_multimedia_player_action (GsdMediaKeysManager *manager,
         return gsd_media_player_key_pressed (manager, key);
 }
 
+static void
+on_xrandr_action_call_finished (GObject             *source_object,
+				GAsyncResult        *res,
+				GsdMediaKeysManager *manager)
+{
+        GError *error = NULL;
+        GVariant *variant;
+        char *action;
+
+        action = g_object_get_data (G_OBJECT (source_object),
+				    "gsd-media-keys-manager-xrandr-action");
+
+	variant = g_dbus_proxy_call_finish (G_DBUS_PROXY (source_object), res, &error);
+
+	g_object_unref (manager->priv->cancellable);
+	manager->priv->cancellable = NULL;
+
+        if (!res) {
+                g_warning ("Unable to call '%s': %s", action, error->message);
+                g_error_free (error);
+        } else {
+		g_variant_unref (variant);
+	}
+
+        g_free (action);
+}
+
+static void
+do_xrandr_action (GsdMediaKeysManager *manager,
+                  const char          *action,
+                  gint64               timestamp)
+{
+        GsdMediaKeysManagerPrivate *priv = manager->priv;
+
+        if (priv->connection == NULL || priv->xrandr_proxy == NULL) {
+                g_warning ("No existing D-Bus connection trying to handle XRANDR keys");
+                return;
+        }
+
+        if (priv->cancellable != NULL) {
+                g_debug ("xrandr action already in flight");
+                return;
+        }
+
+	priv->cancellable = g_cancellable_new ();
+
+        g_object_set_data (G_OBJECT (priv->xrandr_proxy),
+			   "gsd-media-keys-manager-xrandr-action",
+			   g_strdup (action));
+
+	g_dbus_proxy_call (priv->xrandr_proxy,
+			   action,
+			   g_variant_new ("(x)", timestamp),
+			   G_DBUS_CALL_FLAGS_NONE,
+			   -1,
+			   priv->cancellable,
+			   (GAsyncReadyCallback) on_xrandr_action_call_finished,
+			   manager);
+}
+
+static gboolean
+do_video_out_action (GsdMediaKeysManager *manager,
+                     gint64               timestamp)
+{
+        do_xrandr_action (manager, "VideoModeSwitch", timestamp);
+        return FALSE;
+}
+
+static gboolean
+do_video_rotate_action (GsdMediaKeysManager *manager,
+                        gint64               timestamp)
+{
+        do_xrandr_action (manager, "Rotate", timestamp);
+        return FALSE;
+}
+
 static gboolean
 do_action (GsdMediaKeysManager *manager,
-           int                  type)
+           int                  type,
+           gint64               timestamp)
 {
         char *cmd;
         char *path;
@@ -923,8 +1045,8 @@ do_action (GsdMediaKeysManager *manager,
                 do_sound_action (manager, type);
 #endif /* HAVE_PULSE */
                 break;
-        case POWER_KEY:
-                do_exit_action (manager);
+        case LOGOUT_KEY:
+                do_logout_action (manager);
                 break;
         case EJECT_KEY:
                 do_eject_action (manager);
@@ -948,7 +1070,7 @@ do_action (GsdMediaKeysManager *manager,
                 g_free (cmd);
                 break;
         case EMAIL_KEY:
-                do_mail_action (manager);
+                do_url_action (manager, "mailto");
                 break;
         case SCREENSAVER_KEY:
                 if ((cmd = g_find_program_in_path ("gnome-screensaver-command"))) {
@@ -960,10 +1082,10 @@ do_action (GsdMediaKeysManager *manager,
                 g_free (cmd);
                 break;
         case HELP_KEY:
-                do_help_action (manager);
+                do_url_action (manager, "ghelp");
                 break;
         case WWW_KEY:
-                do_www_action (manager, NULL);
+                do_url_action (manager, "http");
                 break;
         case MEDIA_KEY:
                 do_media_action (manager);
@@ -973,21 +1095,32 @@ do_action (GsdMediaKeysManager *manager,
                 break;
         case PLAY_KEY:
                 return do_multimedia_player_action (manager, "Play");
-                break;
         case PAUSE_KEY:
                 return do_multimedia_player_action (manager, "Pause");
-                break;
         case STOP_KEY:
                 return do_multimedia_player_action (manager, "Stop");
-                break;
         case PREVIOUS_KEY:
                 return do_multimedia_player_action (manager, "Previous");
-                break;
         case NEXT_KEY:
                 return do_multimedia_player_action (manager, "Next");
+        case REWIND_KEY:
+                return do_multimedia_player_action (manager, "Rewind");
+        case FORWARD_KEY:
+                return do_multimedia_player_action (manager, "FastForward");
+        case REPEAT_KEY:
+                return do_multimedia_player_action (manager, "Repeat");
+        case RANDOM_KEY:
+                return do_multimedia_player_action (manager, "Shuffle");
+        case VIDEO_OUT_KEY:
+        case VIDEO_OUT2_KEY:
+                do_video_out_action (manager, timestamp);
                 break;
-        default:
+	case ROTATE_VIDEO_KEY:
+		do_video_rotate_action (manager, timestamp);
+		break;
+        case HANDLED_KEYS:
                 g_assert_not_reached ();
+        /* Note, no default so compiler catches missing keys */
         }
 
         return FALSE;
@@ -1046,7 +1179,7 @@ acme_filter_events (GdkXEvent           *xevent,
 
                         manager->priv->current_screen = acme_get_screen_from_event (manager, xany);
 
-                        if (do_action (manager, keys[i].key_type) == FALSE) {
+                        if (do_action (manager, keys[i].key_type, xev->xkey.time) == FALSE) {
                                 return GDK_FILTER_REMOVE;
                         } else {
                                 return GDK_FILTER_CONTINUE;
@@ -1065,12 +1198,9 @@ start_media_keys_idle_cb (GsdMediaKeysManager *manager)
         g_debug ("Starting media_keys manager");
         gnome_settings_profile_start (NULL);
         manager->priv->volume_monitor = g_volume_monitor_get ();
-        manager->priv->conf_client = gconf_client_get_default ();
-
-        gconf_client_add_dir (manager->priv->conf_client,
-                              GCONF_BINDING_DIR,
-                              GCONF_CLIENT_PRELOAD_ONELEVEL,
-                              NULL);
+        manager->priv->settings = g_settings_new (SETTINGS_BINDING_DIR);
+        g_signal_connect (G_OBJECT (manager->priv->settings), "changed",
+                          G_CALLBACK (update_kbd_cb), manager);
 
         init_screens (manager);
         init_kbd (manager);
@@ -1147,20 +1277,9 @@ gsd_media_keys_manager_stop (GsdMediaKeysManager *manager)
                                           manager);
         }
 
-        if (priv->conf_client) {
-                gconf_client_remove_dir (priv->conf_client,
-                                         GCONF_BINDING_DIR,
-                                         NULL);
-
-                for (i = 0; i < HANDLED_KEYS; ++i) {
-                        if (priv->notify[i] != 0) {
-                                gconf_client_notify_remove (priv->conf_client, priv->notify[i]);
-                                priv->notify[i] = 0;
-                        }
-                }
-
-                g_object_unref (priv->conf_client);
-                priv->conf_client = NULL;
+        if (priv->settings) {
+                g_object_unref (priv->settings);
+                priv->settings = NULL;
         }
 
         if (priv->volume_monitor != NULL) {
@@ -1168,11 +1287,24 @@ gsd_media_keys_manager_stop (GsdMediaKeysManager *manager)
                 priv->volume_monitor = NULL;
         }
 
-        if (priv->connection != NULL) {
-                dbus_g_connection_unref (priv->connection);
-                priv->connection = NULL;
+	if (priv->cancellable != NULL) {
+		g_cancellable_cancel (priv->cancellable);
+		g_object_unref (priv->cancellable);
+		priv->cancellable = NULL;
+	}
+
+        if (priv->owner_id > 0) {
+                g_bus_unown_name (priv->owner_id);
+                priv->owner_id = 0;
         }
 
+        if (priv->introspection_data) {
+                g_dbus_node_info_unref (priv->introspection_data);
+                priv->introspection_data = NULL;
+        }
+
+        priv->connection = NULL;
+
         need_flush = FALSE;
         gdk_error_trap_push ();
 
@@ -1271,16 +1403,6 @@ gsd_media_keys_manager_constructor (GType                  type,
         return G_OBJECT (media_keys_manager);
 }
 
-static void
-gsd_media_keys_manager_dispose (GObject *object)
-{
-        GsdMediaKeysManager *media_keys_manager;
-
-        media_keys_manager = GSD_MEDIA_KEYS_MANAGER (object);
-
-        G_OBJECT_CLASS (gsd_media_keys_manager_parent_class)->dispose (object);
-}
-
 static void
 gsd_media_keys_manager_class_init (GsdMediaKeysManagerClass *klass)
 {
@@ -1289,24 +1411,8 @@ gsd_media_keys_manager_class_init (GsdMediaKeysManagerClass *klass)
         object_class->get_property = gsd_media_keys_manager_get_property;
         object_class->set_property = gsd_media_keys_manager_set_property;
         object_class->constructor = gsd_media_keys_manager_constructor;
-        object_class->dispose = gsd_media_keys_manager_dispose;
         object_class->finalize = gsd_media_keys_manager_finalize;
 
-       signals[MEDIA_PLAYER_KEY_PRESSED] =
-               g_signal_new ("media-player-key-pressed",
-                             G_OBJECT_CLASS_TYPE (klass),
-                             G_SIGNAL_RUN_LAST,
-                             G_STRUCT_OFFSET (GsdMediaKeysManagerClass, media_player_key_pressed),
-                             NULL,
-                             NULL,
-                             gsd_marshal_VOID__STRING_STRING,
-                             G_TYPE_NONE,
-                             2,
-                             G_TYPE_STRING,
-                             G_TYPE_STRING);
-
-        dbus_g_object_type_install_info (GSD_TYPE_MEDIA_KEYS_MANAGER, &dbus_glib_gsd_media_keys_manager_object_info);
-
         g_type_class_add_private (klass, sizeof (GsdMediaKeysManagerPrivate));
 }
 
@@ -1314,7 +1420,6 @@ static void
 gsd_media_keys_manager_init (GsdMediaKeysManager *manager)
 {
         manager->priv = GSD_MEDIA_KEYS_MANAGER_GET_PRIVATE (manager);
-
 }
 
 static void
@@ -1332,23 +1437,63 @@ gsd_media_keys_manager_finalize (GObject *object)
         G_OBJECT_CLASS (gsd_media_keys_manager_parent_class)->finalize (object);
 }
 
-static gboolean
+static void
+xrandr_ready_cb (GObject             *source_object,
+		 GAsyncResult        *res,
+		 GsdMediaKeysManager *manager)
+{
+	GError *error = NULL;
+
+	manager->priv->xrandr_proxy = g_dbus_proxy_new_finish (res, &error);
+	if (manager->priv->xrandr_proxy == NULL) {
+		g_warning ("Failed to get proxy for XRandR operations: %s", error->message);
+		g_error_free (error);
+	}
+}
+
+static void
+on_bus_acquired (GDBusConnection     *connection,
+                 const gchar         *name,
+                 GsdMediaKeysManager *manager)
+{
+        guint registration_id;
+
+        registration_id = g_dbus_connection_register_object (connection,
+                                                             GSD_MEDIA_KEYS_DBUS_PATH,
+                                                             manager->priv->introspection_data->interfaces[0],
+                                                             &interface_vtable,
+                                                             manager,
+                                                             NULL,
+                                                             NULL);
+        if (registration_id > 0) {
+                manager->priv->connection = connection;
+
+		g_dbus_proxy_new (manager->priv->connection,
+				  G_DBUS_PROXY_FLAGS_NONE,
+				  NULL,
+				  "org.gnome.SettingsDaemon",
+				  "/org/gnome/SettingsDaemon/XRANDR",
+				  "org.gnome.SettingsDaemon.XRANDR_2",
+				  NULL,
+				  (GAsyncReadyCallback) xrandr_ready_cb,
+				  manager);
+	}
+}
+
+static void
 register_manager (GsdMediaKeysManager *manager)
 {
-        GError *error = NULL;
+        manager->priv->introspection_data = g_dbus_node_info_new_for_xml (introspection_xml, NULL);
+        g_assert (manager->priv->introspection_data != NULL);
 
-        manager->priv->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-        if (manager->priv->connection == NULL) {
-                if (error != NULL) {
-                        g_error ("Error getting session bus: %s", error->message);
-                        g_error_free (error);
-                }
-                return FALSE;
-        }
-
-        dbus_g_connection_register_g_object (manager->priv->connection, GSD_MEDIA_KEYS_DBUS_PATH, G_OBJECT (manager));
-
-        return TRUE;
+        manager->priv->owner_id = g_bus_own_name (G_BUS_TYPE_SESSION,
+                                                  GSD_DBUS_NAME,
+                                                  G_BUS_NAME_OWNER_FLAGS_NONE,
+                                                  (GBusAcquiredCallback) on_bus_acquired,
+                                                  NULL,
+                                                  NULL,
+                                                  manager,
+                                                  NULL);
 }
 
 GsdMediaKeysManager *
@@ -1357,16 +1502,10 @@ gsd_media_keys_manager_new (void)
         if (manager_object != NULL) {
                 g_object_ref (manager_object);
         } else {
-                gboolean res;
-
                 manager_object = g_object_new (GSD_TYPE_MEDIA_KEYS_MANAGER, NULL);
                 g_object_add_weak_pointer (manager_object,
                                            (gpointer *) &manager_object);
-                res = register_manager (manager_object);
-                if (! res) {
-                        g_object_unref (manager_object);
-                        return NULL;
-                }
+                register_manager (manager_object);
         }
 
         return GSD_MEDIA_KEYS_MANAGER (manager_object);
