@@ -34,10 +34,10 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <gconf/gconf-client.h>
 #include <dbus/dbus-glib.h>
 
 #define GNOME_DESKTOP_USE_UNSTABLE_API
@@ -50,20 +50,15 @@
 #include <libnotify/notify.h>
 #endif
 
+#include "gsd-enums.h"
 #include "gnome-settings-profile.h"
 #include "gsd-xrandr-manager.h"
 
-#ifndef HOST_NAME_MAX
-#define HOST_NAME_MAX   255
-#endif
-
 #define GSD_XRANDR_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_XRANDR_MANAGER, GsdXrandrManagerPrivate))
 
-#define CONF_DIR "/apps/gnome_settings_daemon/xrandr"
-#define CONF_KEY_SHOW_NOTIFICATION_ICON (CONF_DIR "/show_notification_icon")
-#define CONF_KEY_TURN_ON_EXTERNAL_MONITORS_AT_STARTUP	(CONF_DIR "/turn_on_external_monitors_at_startup")
-#define CONF_KEY_TURN_ON_LAPTOP_MONITOR_AT_STARTUP	(CONF_DIR "/turn_on_laptop_monitor_at_startup")
-#define CONF_KEY_DEFAULT_CONFIGURATION_FILE             (CONF_DIR "/default_configuration_file")
+#define CONF_DIR "org.gnome.settings-daemon.plugins.xrandr"
+#define CONF_KEY_DEFAULT_MONITORS_SETUP   "default-monitors-setup"
+#define CONF_KEY_DEFAULT_CONFIGURATION_FILE   "default-configuration-file"
 
 #define VIDEO_KEYSYM    "XF86Display"
 #define ROTATE_KEYSYM   "XF86RotateWindows"
@@ -76,9 +71,6 @@
 /* name of the icon files (gsd-xrandr.svg, etc.) */
 #define GSD_XRANDR_ICON_NAME "gsd-xrandr"
 
-/* executable of the control center's display configuration capplet */
-#define GSD_XRANDR_DISPLAY_CAPPLET "gnome-control-center display"
-
 #define GSD_DBUS_PATH "/org/gnome/SettingsDaemon"
 #define GSD_DBUS_NAME "org.gnome.SettingsDaemon"
 #define GSD_XRANDR_DBUS_PATH GSD_DBUS_PATH "/XRANDR"
@@ -88,21 +80,10 @@ struct GsdXrandrManagerPrivate
 {
         DBusGConnection *dbus_connection;
 
-        /* Key code of the XF86Display key (Fn-F7 on Thinkpads, Fn-F4 on HP machines, etc.) */
-        guint switch_video_mode_keycode;
-
-        /* Key code of the XF86RotateWindows key (present on some tablets) */
-        guint rotate_windows_keycode;
-
         GnomeRRScreen *rw_screen;
         gboolean running;
 
-        GtkStatusIcon *status_icon;
-        GtkWidget *popup_menu;
-        GnomeRRConfig *configuration;
-        GnomeRRLabeler *labeler;
-        GConfClient *client;
-        int notify_id;
+        GSettings *settings;
 
         /* fn-F7 status */
         int             current_fn_f7_config;             /* -1 if no configs */
@@ -126,13 +107,13 @@ static void     gsd_xrandr_manager_finalize    (GObject             *object);
 
 static void error_message (GsdXrandrManager *mgr, const char *primary_text, GError *error_to_display, const char *secondary_text);
 
-static void status_icon_popup_menu (GsdXrandrManager *manager, guint button, guint32 timestamp);
-static void run_display_capplet (GtkWidget *widget);
 static void get_allowed_rotations_for_output (GnomeRRConfig *config,
                                               GnomeRRScreen *rr_screen,
                                               GnomeOutputInfo *output,
                                               int *out_num_rotations,
                                               GnomeRRRotation *out_rotations);
+static void handle_fn_f7 (GsdXrandrManager *mgr, guint32 timestamp);
+static void handle_rotate_windows (GsdXrandrManager *mgr, guint32 timestamp);
 
 G_DEFINE_TYPE (GsdXrandrManager, gsd_xrandr_manager, G_TYPE_OBJECT)
 
@@ -493,6 +474,7 @@ user_says_things_are_ok (GsdXrandrManager *manager, GdkWindow *parent_window)
 
         print_countdown_text (&timeout);
 
+        gtk_window_set_icon_name (GTK_WINDOW (timeout.dialog), "preferences-desktop-display");
         gtk_dialog_add_button (GTK_DIALOG (timeout.dialog), _("_Restore Previous Configuration"), GTK_RESPONSE_CANCEL);
         gtk_dialog_add_button (GTK_DIALOG (timeout.dialog), _("_Keep This Configuration"), GTK_RESPONSE_ACCEPT);
         gtk_dialog_set_default_response (GTK_DIALOG (timeout.dialog), GTK_RESPONSE_ACCEPT); /* ah, the optimism */
@@ -626,7 +608,27 @@ gsd_xrandr_manager_2_apply_configuration (GsdXrandrManager *manager,
         return result;
 }
 
-/* We include this after the definition of gsd_xrandr_manager_apply_configuration() so the prototype will already exist */
+/* DBus method for org.gnome.SettingsDaemon.XRANDR_2 VideoModeSwitch; see gsd-xrandr-manager.xml for the interface definition */
+static gboolean
+gsd_xrandr_manager_2_video_mode_switch (GsdXrandrManager *manager,
+					guint32           timestamp,
+					GError          **error)
+{
+	handle_fn_f7 (manager, timestamp);
+	return TRUE;
+}
+
+/* DBus method for org.gnome.SettingsDaemon.XRANDR_2 Rotate; see gsd-xrandr-manager.xml for the interface definition */
+static gboolean
+gsd_xrandr_manager_2_rotate (GsdXrandrManager *manager,
+			     guint32           timestamp,
+			     GError          **error)
+{
+	handle_rotate_windows (manager, timestamp);
+	return TRUE;
+}
+
+/* We include this after the definition of gsd_xrandr_manager_* D-Bus methods so the prototype will already exist */
 #include "gsd-xrandr-manager-glue.h"
 
 static gboolean
@@ -1080,27 +1082,19 @@ static void
 error_message (GsdXrandrManager *mgr, const char *primary_text, GError *error_to_display, const char *secondary_text)
 {
 #ifdef HAVE_LIBNOTIFY
-        GsdXrandrManagerPrivate *priv = mgr->priv;
         NotifyNotification *notification;
 
         g_assert (error_to_display == NULL || secondary_text == NULL);
 
-        if (priv->status_icon)
-                notification = notify_notification_new_with_status_icon (primary_text,
-                                                                         error_to_display ? error_to_display->message : secondary_text,
-                                                                         GSD_XRANDR_ICON_NAME,
-                                                                         priv->status_icon);
-        else
-                notification = notify_notification_new (primary_text,
-                                                        error_to_display ? error_to_display->message : secondary_text,
-                                                        GSD_XRANDR_ICON_NAME,
-                                                        NULL);
+        notification = notify_notification_new (primary_text,
+                                                error_to_display ? error_to_display->message : secondary_text,
+                                                GSD_XRANDR_ICON_NAME);
 
         notify_notification_show (notification, NULL); /* NULL-GError */
 #else
         GtkWidget *dialog;
 
-	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+        dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
                                          "%s", primary_text);
         gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s",
                                                   error_to_display ? error_to_display->message : secondary_text);
@@ -1229,7 +1223,6 @@ get_laptop_output_info (GnomeRRScreen *screen, GnomeRRConfig *config)
         }
 
         return NULL;
-
 }
 
 static GnomeRRRotation
@@ -1322,44 +1315,6 @@ out:
         gnome_rr_config_free (current);
 }
 
-static GdkFilterReturn
-event_filter (GdkXEvent           *xevent,
-              GdkEvent            *event,
-              gpointer             data)
-{
-        GsdXrandrManager *manager = data;
-        XEvent *xev = (XEvent *) xevent;
-
-        if (!manager->priv->running)
-                return GDK_FILTER_CONTINUE;
-
-        /* verify we have a key event */
-        if (xev->xany.type != KeyPress && xev->xany.type != KeyRelease)
-                return GDK_FILTER_CONTINUE;
-
-        if (xev->xany.type == KeyPress) {
-                if (xev->xkey.keycode == manager->priv->switch_video_mode_keycode)
-                        handle_fn_f7 (manager, xev->xkey.time);
-                else if (xev->xkey.keycode == manager->priv->rotate_windows_keycode)
-                        handle_rotate_windows (manager, xev->xkey.time);
-
-                return GDK_FILTER_CONTINUE;
-        }
-
-        return GDK_FILTER_CONTINUE;
-}
-
-static void
-refresh_tray_icon_menu_if_active (GsdXrandrManager *manager, guint32 timestamp)
-{
-        GsdXrandrManagerPrivate *priv = manager->priv;
-
-        if (priv->popup_menu) {
-                gtk_menu_shell_cancel (GTK_MENU_SHELL (priv->popup_menu)); /* status_icon_popup_menu_selection_done_cb() will free everything */
-                status_icon_popup_menu (manager, 0, timestamp);
-        }
-}
-
 static void
 auto_configure_outputs (GsdXrandrManager *manager, guint32 timestamp)
 {
@@ -1483,29 +1438,6 @@ auto_configure_outputs (GsdXrandrManager *manager, guint32 timestamp)
 
         g_list_free (just_turned_on);
         gnome_rr_config_free (config);
-
-        /* Finally, even though we did a best-effort job in sanitizing the
-         * outputs, we don't know the physical layout of the monitors.  We'll
-         * start the display capplet so that the user can tweak things to his
-         * liking.
-         */
-
-#if 0
-        /* FIXME: This is disabled for now.  The capplet is not a single-instance application.
-         * If you do this:
-         *
-         *   1. Start the display capplet
-         *
-         *   2. Plug an extra monitor
-         *
-         *   3. Hit the "Detect displays" button
-         *
-         * Then we will get a RANDR event because X re-probes the outputs.  We don't want to
-         * start up a second display capplet right there!
-         */
-
-        run_display_capplet (NULL);
-#endif
 }
 
 static void
@@ -1604,226 +1536,9 @@ on_randr_event (GnomeRRScreen *screen, gpointer data)
         /* poke gnome-color-manager */
         apply_color_profiles ();
 
-        refresh_tray_icon_menu_if_active (manager, MAX (change_timestamp, config_timestamp));
-
         log_close ();
 }
 
-static void
-run_display_capplet (GtkWidget *widget)
-{
-        GdkScreen *screen;
-        GError *error;
-
-        if (widget)
-                screen = gtk_widget_get_screen (widget);
-        else
-                screen = gdk_screen_get_default ();
-
-        error = NULL;
-        if (!gdk_spawn_command_line_on_screen (screen, GSD_XRANDR_DISPLAY_CAPPLET, &error)) {
-		GtkWidget *dialog;
-
-		dialog = gtk_message_dialog_new_with_markup (NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-                                                             "<span weight=\"bold\" size=\"larger\">"
-                                                             "Display configuration could not be run"
-                                                             "</span>\n\n"
-                                                             "%s", error->message);
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
-
-		g_error_free (error);
-        }
-}
-
-static void
-popup_menu_configure_display_cb (GtkMenuItem *item, gpointer data)
-{
-        run_display_capplet (GTK_WIDGET (item));
-}
-
-static void
-status_icon_popup_menu_selection_done_cb (GtkMenuShell *menu_shell, gpointer data)
-{
-        GsdXrandrManager *manager = GSD_XRANDR_MANAGER (data);
-        struct GsdXrandrManagerPrivate *priv = manager->priv;
-
-        gtk_widget_destroy (priv->popup_menu);
-        priv->popup_menu = NULL;
-
-        gnome_rr_labeler_hide (priv->labeler);
-        g_object_unref (priv->labeler);
-        priv->labeler = NULL;
-
-        gnome_rr_config_free (priv->configuration);
-        priv->configuration = NULL;
-}
-
-#define OUTPUT_TITLE_ITEM_BORDER 2
-#define OUTPUT_TITLE_ITEM_PADDING 4
-
-/* This is an expose-event hander for the title label for each GnomeRROutput.
- * We want each title to have a colored background, so we paint that background, then
- * return FALSE to let GtkLabel expose itself (i.e. paint the label's text), and then
- * we have a signal_connect_after handler as well.  See the comments below
- * to see why that "after" handler is needed.
- */
-static gboolean
-output_title_label_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
-{
-        GsdXrandrManager *manager = GSD_XRANDR_MANAGER (data);
-        struct GsdXrandrManagerPrivate *priv = manager->priv;
-        GnomeOutputInfo *output;
-        GdkColor color;
-        cairo_t *cr;
-        GtkAllocation allocation;
-
-        g_assert (GTK_IS_LABEL (widget));
-
-        output = g_object_get_data (G_OBJECT (widget), "output");
-        g_assert (output != NULL);
-
-        g_assert (priv->labeler != NULL);
-
-        /* Draw a black rectangular border, filled with the color that corresponds to this output */
-
-        gnome_rr_labeler_get_color_for_output (priv->labeler, output, &color);
-
-        cr = gdk_cairo_create (gtk_widget_get_window (widget));
-
-        cairo_set_source_rgb (cr, 0, 0, 0);
-        cairo_set_line_width (cr, OUTPUT_TITLE_ITEM_BORDER);
-        gtk_widget_get_allocation (widget, &allocation);
-        cairo_rectangle (cr,
-                         allocation.x + OUTPUT_TITLE_ITEM_BORDER / 2.0,
-                         allocation.y + OUTPUT_TITLE_ITEM_BORDER / 2.0,
-                         allocation.width - OUTPUT_TITLE_ITEM_BORDER,
-                         allocation.height - OUTPUT_TITLE_ITEM_BORDER);
-        cairo_stroke (cr);
-
-        gdk_cairo_set_source_color (cr, &color);
-        cairo_rectangle (cr,
-                         allocation.x + OUTPUT_TITLE_ITEM_BORDER,
-                         allocation.y + OUTPUT_TITLE_ITEM_BORDER,
-                         allocation.width - 2 * OUTPUT_TITLE_ITEM_BORDER,
-                         allocation.height - 2 * OUTPUT_TITLE_ITEM_BORDER);
-
-        cairo_fill (cr);
-
-        /* We want the label to always show up as if it were sensitive
-         * ("style->fg[GTK_STATE_NORMAL]"), even though the label is insensitive
-         * due to being inside an insensitive menu item.  So, here we have a
-         * HACK in which we frob the label's state directly.  GtkLabel's expose
-         * handler will be run after this function, so it will think that the
-         * label is in GTK_STATE_NORMAL.  We reset the label's state back to
-         * insensitive in output_title_label_after_expose_event_cb().
-         *
-         * Yay for fucking with GTK+'s internals.
-         */
-
-        gtk_widget_set_state (widget, GTK_STATE_NORMAL);
-
-        return FALSE;
-}
-
-/* See the comment in output_title_event_box_expose_event_cb() about this funny label widget */
-static gboolean
-output_title_label_after_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
-{
-        g_assert (GTK_IS_LABEL (widget));
-        gtk_widget_set_state (widget, GTK_STATE_INSENSITIVE);
-
-        return FALSE;
-}
-
-static void
-title_item_size_allocate_cb (GtkWidget *widget, GtkAllocation *allocation, gpointer data)
-{
-        /* When GtkMenu does size_request on its items, it asks them for their "toggle size",
-         * which will be non-zero when there are check/radio items.  GtkMenu remembers
-         * the largest of those sizes.  During the size_allocate pass, GtkMenu calls
-         * gtk_menu_item_toggle_size_allocate() with that value, to tell the menu item
-         * that it should later paint its child a bit to the right of its edge.
-         *
-         * However, we want the "title" menu items for each RANDR output to span the *whole*
-         * allocation of the menu item, not just the "allocation minus toggle" area.
-         *
-         * So, we let the menu item size_allocate itself as usual, but this
-         * callback gets run afterward.  Here we hack a toggle size of 0 into
-         * the menu item, and size_allocate it by hand *again*.  We also need to
-         * avoid recursing into this function.
-         */
-
-        g_assert (GTK_IS_MENU_ITEM (widget));
-
-        gtk_menu_item_toggle_size_allocate (GTK_MENU_ITEM (widget), 0);
-
-        g_signal_handlers_block_by_func (widget, title_item_size_allocate_cb, NULL);
-
-        /* Sigh. There is no way to turn on GTK_ALLOC_NEEDED outside of GTK+
-         * itself; also, since calling size_allocate on a widget with the same
-         * allcation is a no-op, we need to allocate with a "different" size
-         * first.
-         */
-
-        allocation->width++;
-        gtk_widget_size_allocate (widget, allocation);
-
-        allocation->width--;
-        gtk_widget_size_allocate (widget, allocation);
-
-        g_signal_handlers_unblock_by_func (widget, title_item_size_allocate_cb, NULL);
-}
-
-static GtkWidget *
-make_menu_item_for_output_title (GsdXrandrManager *manager, GnomeOutputInfo *output)
-{
-        GtkWidget *item;
-        GtkWidget *label;
-        char *str;
-	GdkColor black = { 0, 0, 0, 0 };
-
-        item = gtk_menu_item_new ();
-
-        g_signal_connect (item, "size-allocate",
-                          G_CALLBACK (title_item_size_allocate_cb), NULL);
-
-        str = g_markup_printf_escaped ("<b>%s</b>", output->display_name);
-        label = gtk_label_new (NULL);
-        gtk_label_set_markup (GTK_LABEL (label), str);
-        g_free (str);
-
-	/* Make the label explicitly black.  We don't want it to follow the
-	 * theme's colors, since the label is always shown against a light
-	 * pastel background.  See bgo#556050
-	 */
-	gtk_widget_modify_fg (label, gtk_widget_get_state (label), &black);
-
-        /* Add padding around the label to fit the box that we'll draw for color-coding */
-        gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-        gtk_misc_set_padding (GTK_MISC (label),
-                              OUTPUT_TITLE_ITEM_BORDER + OUTPUT_TITLE_ITEM_PADDING,
-                              OUTPUT_TITLE_ITEM_BORDER + OUTPUT_TITLE_ITEM_PADDING);
-
-        gtk_container_add (GTK_CONTAINER (item), label);
-
-        /* We want to paint a colored box as the background of the label, so we connect
-         * to its expose-event signal.  See the comment in *** to see why need to connect
-         * to the label both 'before' and 'after'.
-         */
-        g_signal_connect (label, "expose-event",
-                          G_CALLBACK (output_title_label_expose_event_cb), manager);
-        g_signal_connect_after (label, "expose-event",
-                                G_CALLBACK (output_title_label_after_expose_event_cb), manager);
-
-        g_object_set_data (G_OBJECT (label), "output", output);
-
-        gtk_widget_set_sensitive (item, FALSE); /* the title is not selectable */
-        gtk_widget_show_all (item);
-
-        return item;
-}
-
 static void
 get_allowed_rotations_for_output (GnomeRRConfig *config,
                                   GnomeRRScreen *rr_screen,
@@ -1863,315 +1578,18 @@ get_allowed_rotations_for_output (GnomeRRConfig *config,
         }
 }
 
-static void
-add_unsupported_rotation_item (GsdXrandrManager *manager)
-{
-        struct GsdXrandrManagerPrivate *priv = manager->priv;
-        GtkWidget *item;
-        GtkWidget *label;
-        gchar *markup;
-
-        item = gtk_menu_item_new ();
-
-        label = gtk_label_new (NULL);
-        markup = g_strdup_printf ("<i>%s</i>", _("Rotation not supported"));
-        gtk_label_set_markup (GTK_LABEL (label), markup);
-        g_free (markup);
-        gtk_container_add (GTK_CONTAINER (item), label);
-
-        gtk_widget_show_all (item);
-        gtk_menu_shell_append (GTK_MENU_SHELL (priv->popup_menu), item);
-}
-
-static void
-ensure_current_configuration_is_saved (void)
-{
-        GnomeRRScreen *rr_screen;
-        GnomeRRConfig *rr_config;
-
-        /* Normally, gnome_rr_config_save() creates a backup file based on the
-         * old monitors.xml.  However, if *that* file didn't exist, there is
-         * nothing from which to create a backup.  So, here we'll save the
-         * current/unchanged configuration and then let our caller call
-         * gnome_rr_config_save() again with the new/changed configuration, so
-         * that there *will* be a backup file in the end.
-         */
-
-        rr_screen = gnome_rr_screen_new (gdk_screen_get_default (), NULL, NULL, NULL); /* NULL-GError */
-        if (!rr_screen)
-                return;
-
-        rr_config = gnome_rr_config_new_current (rr_screen);
-        gnome_rr_config_save (rr_config, NULL); /* NULL-GError */
-
-        gnome_rr_config_free (rr_config);
-        gnome_rr_screen_destroy (rr_screen);
-}
-
-static void
-output_rotation_item_activate_cb (GtkCheckMenuItem *item, gpointer data)
-{
-        GsdXrandrManager *manager = GSD_XRANDR_MANAGER (data);
-        struct GsdXrandrManagerPrivate *priv = manager->priv;
-        GnomeOutputInfo *output;
-        GnomeRRRotation rotation;
-        GError *error;
-
-	/* Not interested in deselected items */
-	if (!gtk_check_menu_item_get_active (item))
-		return;
-
-        ensure_current_configuration_is_saved ();
-
-        output = g_object_get_data (G_OBJECT (item), "output");
-        rotation = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "rotation"));
-
-        output->rotation = rotation;
-
-        error = NULL;
-        if (!gnome_rr_config_save (priv->configuration, &error)) {
-                error_message (manager, _("Could not save monitor configuration"), error, NULL);
-                if (error)
-                        g_error_free (error);
-
-                return;
-        }
-
-        try_to_apply_intended_configuration (manager, NULL, gtk_get_current_event_time (), NULL); /* NULL-GError */
-}
-
-static void
-add_items_for_rotations (GsdXrandrManager *manager, GnomeOutputInfo *output, GnomeRRRotation allowed_rotations)
-{
-        typedef struct {
-                GnomeRRRotation	rotation;
-                const char *	name;
-        } RotationInfo;
-        static const RotationInfo rotations[] = {
-                { GNOME_RR_ROTATION_0, N_("Normal") },
-                { GNOME_RR_ROTATION_90, N_("Left") },
-                { GNOME_RR_ROTATION_270, N_("Right") },
-                { GNOME_RR_ROTATION_180, N_("Upside Down") },
-                /* We don't allow REFLECT_X or REFLECT_Y for now, as gnome-display-properties doesn't allow them, either */
-        };
-
-        struct GsdXrandrManagerPrivate *priv = manager->priv;
-        int i;
-        GSList *group;
-        GtkWidget *active_item;
-        gulong active_item_activate_id;
-
-        group = NULL;
-        active_item = NULL;
-        active_item_activate_id = 0;
-
-        for (i = 0; i < G_N_ELEMENTS (rotations); i++) {
-                GnomeRRRotation rot;
-                GtkWidget *item;
-                gulong activate_id;
-
-                rot = rotations[i].rotation;
-
-                if ((allowed_rotations & rot) == 0) {
-                        /* don't display items for rotations which are
-                         * unavailable.  Their availability is not under the
-                         * user's control, anyway.
-                         */
-                        continue;
-                }
-
-                item = gtk_radio_menu_item_new_with_label (group, _(rotations[i].name));
-                gtk_widget_show_all (item);
-                gtk_menu_shell_append (GTK_MENU_SHELL (priv->popup_menu), item);
-
-                g_object_set_data (G_OBJECT (item), "output", output);
-                g_object_set_data (G_OBJECT (item), "rotation", GINT_TO_POINTER (rot));
-
-                activate_id = g_signal_connect (item, "activate",
-                                                G_CALLBACK (output_rotation_item_activate_cb), manager);
-
-                group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (item));
-
-                if (rot == output->rotation) {
-                        active_item = item;
-                        active_item_activate_id = activate_id;
-                }
-        }
-
-        if (active_item) {
-                /* Block the signal temporarily so our callback won't be called;
-                 * we are just setting up the UI.
-                 */
-                g_signal_handler_block (active_item, active_item_activate_id);
-
-                gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (active_item), TRUE);
-
-                g_signal_handler_unblock (active_item, active_item_activate_id);
-        }
-
-}
-
-static void
-add_rotation_items_for_output (GsdXrandrManager *manager, GnomeOutputInfo *output)
-{
-        struct GsdXrandrManagerPrivate *priv = manager->priv;
-        int num_rotations;
-        GnomeRRRotation rotations;
-
-        get_allowed_rotations_for_output (priv->configuration, priv->rw_screen, output, &num_rotations, &rotations);
-
-        if (num_rotations == 1)
-                add_unsupported_rotation_item (manager);
-        else
-                add_items_for_rotations (manager, output, rotations);
-}
-
-static void
-add_menu_items_for_output (GsdXrandrManager *manager, GnomeOutputInfo *output)
-{
-        struct GsdXrandrManagerPrivate *priv = manager->priv;
-        GtkWidget *item;
-
-        item = make_menu_item_for_output_title (manager, output);
-        gtk_menu_shell_append (GTK_MENU_SHELL (priv->popup_menu), item);
-
-        add_rotation_items_for_output (manager, output);
-}
-
-static void
-add_menu_items_for_outputs (GsdXrandrManager *manager)
-{
-        struct GsdXrandrManagerPrivate *priv = manager->priv;
-        int i;
-
-        for (i = 0; priv->configuration->outputs[i] != NULL; i++) {
-                if (priv->configuration->outputs[i]->connected)
-                        add_menu_items_for_output (manager, priv->configuration->outputs[i]);
-        }
-}
-
-static void
-status_icon_popup_menu (GsdXrandrManager *manager, guint button, guint32 timestamp)
-{
-        struct GsdXrandrManagerPrivate *priv = manager->priv;
-        GtkWidget *item;
-
-        g_assert (priv->configuration == NULL);
-        priv->configuration = gnome_rr_config_new_current (priv->rw_screen);
-
-        g_assert (priv->labeler == NULL);
-        priv->labeler = gnome_rr_labeler_new (priv->configuration);
-
-        g_assert (priv->popup_menu == NULL);
-        priv->popup_menu = gtk_menu_new ();
-
-        add_menu_items_for_outputs (manager);
-
-        item = gtk_separator_menu_item_new ();
-        gtk_widget_show (item);
-        gtk_menu_shell_append (GTK_MENU_SHELL (priv->popup_menu), item);
-
-        item = gtk_menu_item_new_with_mnemonic (_("_Configure Display Settings…"));
-        g_signal_connect (item, "activate",
-                          G_CALLBACK (popup_menu_configure_display_cb), manager);
-        gtk_widget_show (item);
-        gtk_menu_shell_append (GTK_MENU_SHELL (priv->popup_menu), item);
-
-        g_signal_connect (priv->popup_menu, "selection-done",
-                          G_CALLBACK (status_icon_popup_menu_selection_done_cb), manager);
-
-        gtk_menu_popup (GTK_MENU (priv->popup_menu), NULL, NULL,
-                        gtk_status_icon_position_menu,
-                        priv->status_icon, button, timestamp);
-}
-
-static void
-status_icon_activate_cb (GtkStatusIcon *status_icon, gpointer data)
-{
-        GsdXrandrManager *manager = GSD_XRANDR_MANAGER (data);
-
-        /* Suck; we don't get a proper button/timestamp */
-        status_icon_popup_menu (manager, 0, gtk_get_current_event_time ());
-}
-
-static void
-status_icon_popup_menu_cb (GtkStatusIcon *status_icon, guint button, guint32 timestamp, gpointer data)
-{
-        GsdXrandrManager *manager = GSD_XRANDR_MANAGER (data);
-
-        status_icon_popup_menu (manager, button, timestamp);
-}
-
-static void
-status_icon_start (GsdXrandrManager *manager)
-{
-        struct GsdXrandrManagerPrivate *priv = manager->priv;
-
-        /* Ideally, we should detect if we are on a tablet and only display
-         * the icon in that case.
-         */
-        if (!priv->status_icon) {
-                priv->status_icon = gtk_status_icon_new_from_icon_name (GSD_XRANDR_ICON_NAME);
-                gtk_status_icon_set_tooltip_text (priv->status_icon, _("Configure display settings"));
-
-                g_signal_connect (priv->status_icon, "activate",
-                                  G_CALLBACK (status_icon_activate_cb), manager);
-                g_signal_connect (priv->status_icon, "popup-menu",
-                                  G_CALLBACK (status_icon_popup_menu_cb), manager);
-        }
-}
-
-static void
-status_icon_stop (GsdXrandrManager *manager)
-{
-        struct GsdXrandrManagerPrivate *priv = manager->priv;
-
-        if (priv->status_icon) {
-                g_signal_handlers_disconnect_by_func (
-                        priv->status_icon, G_CALLBACK (status_icon_activate_cb), manager);
-                g_signal_handlers_disconnect_by_func (
-                        priv->status_icon, G_CALLBACK (status_icon_popup_menu_cb), manager);
-
-                /* hide the icon before unreffing it; otherwise we will leak
-                   whitespace in the notification area due to a bug in there */
-                gtk_status_icon_set_visible (priv->status_icon, FALSE);
-                g_object_unref (priv->status_icon);
-                priv->status_icon = NULL;
-        }
-}
-
-static void
-start_or_stop_icon (GsdXrandrManager *manager)
-{
-        if (gconf_client_get_bool (manager->priv->client, CONF_KEY_SHOW_NOTIFICATION_ICON, NULL)) {
-                status_icon_start (manager);
-        }
-        else {
-                status_icon_stop (manager);
-        }
-}
-
-static void
-on_config_changed (GConfClient          *client,
-                   guint                 cnxn_id,
-                   GConfEntry           *entry,
-                   GsdXrandrManager *manager)
-{
-        if (strcmp (entry->key, CONF_KEY_SHOW_NOTIFICATION_ICON) == 0)
-                start_or_stop_icon (manager);
-}
-
 static gboolean
 apply_intended_configuration (GsdXrandrManager *manager, const char *intended_filename, guint32 timestamp)
 {
         GError *my_error;
-	gboolean result;
+        gboolean result;
 
         my_error = NULL;
-        result = apply_configuration_from_filename (manager, intended_filename, FALSE, timestamp, &my_error);
+        result = apply_configuration_from_filename (manager, intended_filename, TRUE, timestamp, &my_error);
         if (!result) {
                 if (my_error) {
-                        if (!g_error_matches (my_error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
+                        if (!g_error_matches (my_error, G_FILE_ERROR, G_FILE_ERROR_NOENT) &&
+                            !g_error_matches (my_error, GNOME_RR_ERROR, GNOME_RR_ERROR_NO_MATCHING_CONFIG))
                                 error_message (manager, _("Could not apply the stored configuration for monitors"), my_error, NULL);
 
                         g_error_free (my_error);
@@ -2184,29 +1602,30 @@ apply_intended_configuration (GsdXrandrManager *manager, const char *intended_fi
 static void
 apply_default_boot_configuration (GsdXrandrManager *mgr, guint32 timestamp)
 {
-	GsdXrandrManagerPrivate *priv = mgr->priv;
-	GnomeRRScreen *screen = priv->rw_screen;
-   	GnomeRRConfig *config;
-   	gboolean turn_on_external, turn_on_laptop;
+        GsdXrandrManagerPrivate *priv = mgr->priv;
+        GnomeRRScreen *screen = priv->rw_screen;
+        GnomeRRConfig *config;
+        GsdXrandrBootBehaviour boot;
 
-        turn_on_external =
-                gconf_client_get_bool (mgr->priv->client, CONF_KEY_TURN_ON_EXTERNAL_MONITORS_AT_STARTUP, NULL);
-        turn_on_laptop =
-                gconf_client_get_bool (mgr->priv->client, CONF_KEY_TURN_ON_LAPTOP_MONITOR_AT_STARTUP, NULL);
+        boot = g_settings_get_enum (priv->settings, CONF_KEY_DEFAULT_MONITORS_SETUP);
 
-	if (turn_on_external && turn_on_laptop)
+        switch (boot) {
+	case GSD_XRANDR_BOOT_BEHAVIOUR_DO_NOTHING:
+		return;
+	case GSD_XRANDR_BOOT_BEHAVIOUR_CLONE:
 		config = make_clone_setup (screen);
-	else if (!turn_on_external && turn_on_laptop)
-		config = make_laptop_setup (screen);
-	else if (turn_on_external && !turn_on_laptop)
+		break;
+	case GSD_XRANDR_BOOT_BEHAVIOUR_DOCK:
 		config = make_other_setup (screen);
-	else
-		config = make_laptop_setup (screen);
+		break;
+	default:
+		g_assert_not_reached ();
+	}
 
-        if (config) {
-                apply_configuration_and_display_error (mgr, config, timestamp);
-                gnome_rr_config_free (config);
-        }
+	if (config) {
+		apply_configuration_and_display_error (mgr, config, timestamp);
+		gnome_rr_config_free (config);
+	}
 }
 
 static gboolean
@@ -2273,7 +1692,7 @@ apply_default_configuration_from_file (GsdXrandrManager *manager, guint32 timest
         char *default_config_filename;
         gboolean result;
 
-        default_config_filename = gconf_client_get_string (priv->client, CONF_KEY_DEFAULT_CONFIGURATION_FILE, NULL);
+        default_config_filename = g_settings_get_string (priv->settings, CONF_KEY_DEFAULT_CONFIGURATION_FILE);
         if (!default_config_filename)
                 return FALSE;
 
@@ -2308,58 +1727,16 @@ gsd_xrandr_manager_start (GsdXrandrManager *manager,
         log_screen (manager->priv->rw_screen);
 
         manager->priv->running = TRUE;
-        manager->priv->client = gconf_client_get_default ();
-
-        g_assert (manager->priv->notify_id == 0);
-
-        gconf_client_add_dir (manager->priv->client, CONF_DIR,
-                              GCONF_CLIENT_PRELOAD_ONELEVEL,
-                              NULL);
-
-        manager->priv->notify_id =
-                gconf_client_notify_add (
-                        manager->priv->client, CONF_DIR,
-                        (GConfClientNotifyFunc)on_config_changed,
-                        manager, NULL, NULL);
-
-        if (manager->priv->switch_video_mode_keycode) {
-                gdk_error_trap_push ();
-
-                XGrabKey (gdk_x11_get_default_xdisplay(),
-                          manager->priv->switch_video_mode_keycode, AnyModifier,
-                          gdk_x11_get_default_root_xwindow(),
-                          True, GrabModeAsync, GrabModeAsync);
-
-                gdk_flush ();
-                gdk_error_trap_pop ();
-        }
-
-        if (manager->priv->rotate_windows_keycode) {
-                gdk_error_trap_push ();
-
-                XGrabKey (gdk_x11_get_default_xdisplay(),
-                          manager->priv->rotate_windows_keycode, AnyModifier,
-                          gdk_x11_get_default_root_xwindow(),
-                          True, GrabModeAsync, GrabModeAsync);
-
-                gdk_flush ();
-                gdk_error_trap_pop ();
-        }
+        manager->priv->settings = g_settings_new (CONF_DIR);
 
         show_timestamps_dialog (manager, "Startup");
         if (!apply_stored_configuration_at_startup (manager, GDK_CURRENT_TIME)) /* we don't have a real timestamp at startup anyway */
                 if (!apply_default_configuration_from_file (manager, GDK_CURRENT_TIME))
-                        apply_default_boot_configuration (manager, GDK_CURRENT_TIME);
+			apply_default_boot_configuration (manager, GDK_CURRENT_TIME);
 
         log_msg ("State of screen after initial configuration:\n");
         log_screen (manager->priv->rw_screen);
 
-        gdk_window_add_filter (gdk_get_default_root_window(),
-                               (GdkFilterFunc)event_filter,
-                               manager);
-
-        start_or_stop_icon (manager);
-
         log_close ();
 
         gnome_settings_profile_end (NULL);
@@ -2374,41 +1751,9 @@ gsd_xrandr_manager_stop (GsdXrandrManager *manager)
 
         manager->priv->running = FALSE;
 
-        if (manager->priv->switch_video_mode_keycode) {
-                gdk_error_trap_push ();
-
-                XUngrabKey (gdk_x11_get_default_xdisplay(),
-                            manager->priv->switch_video_mode_keycode, AnyModifier,
-                            gdk_x11_get_default_root_xwindow());
-
-                gdk_error_trap_pop ();
-        }
-
-        if (manager->priv->rotate_windows_keycode) {
-                gdk_error_trap_push ();
-
-                XUngrabKey (gdk_x11_get_default_xdisplay(),
-                            manager->priv->rotate_windows_keycode, AnyModifier,
-                            gdk_x11_get_default_root_xwindow());
-
-                gdk_error_trap_pop ();
-        }
-
-        gdk_window_remove_filter (gdk_get_default_root_window (),
-                                  (GdkFilterFunc) event_filter,
-                                  manager);
-
-        if (manager->priv->notify_id != 0) {
-                gconf_client_remove_dir (manager->priv->client,
-                                         CONF_DIR, NULL);
-                gconf_client_notify_remove (manager->priv->client,
-                                            manager->priv->notify_id);
-                manager->priv->notify_id = 0;
-        }
-
-        if (manager->priv->client != NULL) {
-                g_object_unref (manager->priv->client);
-                manager->priv->client = NULL;
+        if (manager->priv->settings != NULL) {
+                g_object_unref (manager->priv->settings);
+                manager->priv->settings = NULL;
         }
 
         if (manager->priv->rw_screen != NULL) {
@@ -2421,8 +1766,6 @@ gsd_xrandr_manager_stop (GsdXrandrManager *manager)
                 manager->priv->dbus_connection = NULL;
         }
 
-        status_icon_stop (manager);
-
         log_open ();
         log_msg ("STOPPING XRANDR PLUGIN\n------------------------------------------------------------\n");
         log_close ();
@@ -2505,26 +1848,11 @@ gsd_xrandr_manager_class_init (GsdXrandrManagerClass *klass)
         g_type_class_add_private (klass, sizeof (GsdXrandrManagerPrivate));
 }
 
-static guint
-get_keycode_for_keysym_name (const char *name)
-{
-        Display *dpy;
-        guint keyval;
-
-        dpy = gdk_x11_get_default_xdisplay ();
-
-        keyval = gdk_keyval_from_name (name);
-        return XKeysymToKeycode (dpy, keyval);
-}
-
 static void
 gsd_xrandr_manager_init (GsdXrandrManager *manager)
 {
         manager->priv = GSD_XRANDR_MANAGER_GET_PRIVATE (manager);
 
-        manager->priv->switch_video_mode_keycode = get_keycode_for_keysym_name (VIDEO_KEYSYM);
-        manager->priv->rotate_windows_keycode = get_keycode_for_keysym_name (ROTATE_KEYSYM);
-
         manager->priv->current_fn_f7_config = -1;
         manager->priv->fn_f7_configs = NULL;
 }
